/*
stattest - tests for randomness
Copyright (C) 2003  Roland Siegert, Andreas Stricker

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
*/

/** \file
 * \brief Module fileops: Read file into buffer
 *
 * Read a file into local buffer
 *
 * \author Andreas Stricker
 * \author Roland Siegert
 * \version $Id: fileops.c,v 1.5 2003/05/15 07:54:54 stricand Exp $
 */
 
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdlib.h>
 
#include "fileops.h"
#include "stattest.h"

/** \brief Fill Buffer with content of file, specified with "filename".
 * Open the file and fill buffer with content. Note that the file size
 * is not restricted because this function read never more than MAX_BUF_SIZE
 * bytes from file.
 * \param filename name of the file
 * \param buffer pointer to a buffer_t, later points to the data read
 * \param maxsize only read maxsize data
 * \return 0 if error or number of Bytes read from file (buffersize).
 */
size_t fill_buffer_from_file(char *filename, buffer_t *buffer, size_t maxsize) {
	size_t bufsize, readsize;
	FILE *filehandle;
	struct stat filestat;

	// get some information from file
	if (stat(filename, &filestat) < 0) {
		printf(MSG_ERR_FILE_NOT_ACCESSIBLE, filename);
		*buffer = NULL;
		return 0;
	}

	bufsize = filestat.st_size;				// get file size
	if (bufsize > MAX_BUF_SIZE) {
		printf(MSG_INFO_FILE_TOO_BIG, MAX_BUF_SIZE);
		bufsize = MAX_BUF_SIZE;
	}
	if ((maxsize != 0) && (bufsize > maxsize)) {	// user choose a size
		bufsize = maxsize;
	}
	*buffer = (buffer_t) malloc(bufsize);	// allocate memory
	if (*buffer == NULL) {
		printf(MSG_ERR_NO_ENOUGH_MEMORY);
		*buffer = NULL;
		return 0;
	}

	filehandle = fopen(filename, "r");		// open file
	if (filehandle == NULL) {
		printf(MSG_ERR_CANT_OPEN_FILE);
		free(*buffer); *buffer = NULL;
		return 0;
	}
	readsize = fread(*buffer, sizeof(block_t), bufsize, filehandle);	// read file
	if (readsize < bufsize) {
		printf(MSG_ERR_CANT_READ_WHOLE_FILE, bufsize-readsize);
		free(*buffer); *buffer = NULL;
		return 0;
	}

	fclose(filehandle);
	return readsize;
} // fill_buffer_from_file

