/*
stattest - tests for randomness
Copyright (C) 2003  Roland Siegert, Andreas Stricker

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
*/

/** \file
 * \brief Universal statistical test T_U
 *
 * Original Pascal Code converted to C
 *
 * \author Ueli M. Maurer
 * \author Andreas Stricker
 * \author Roland Siegert
 * \version $Id: universal.h,v 1.10 2003/05/16 14:07:33 stricand Exp $
 */
 
#ifndef UNIVERSAL_H_9FA7
#define UNIVERSAL_H_9FA7

#include <stdlib.h>
#include "stattest.h"

/** Datatype of test result */
struct test_params_t {
	double ftu;				//!< the test resut fTU closly related to entropy
	size_t parameter_Q;		//!< the test parameter Q
	size_t parameter_K;		//!< the test parameter K
	size_t parameter_L;		//!< the test parameter L
	double rejection_rate;	//!< choosen rejection rate
	double t1;				//!< lower border for test result
	double t2;				//!< higher border for test result
	bool accepted;			//!< test result (accepted = true, rejected = false)
};

/**
 * \brief The universal statistical test is performed here.
 * The function tests the whole buffer with block_size size and write
 * the results to the structure test_results.
 * \param buffer Buffer with random data of size block_size
 * \param block_size size of buffer
 * \param test_results pointer to struct of type test_params_t, filled
 *         with the result of test and also returned by this function
 * \param block_size_L Size of a block (8 or 16)
 * \return result of test pointer to the same structure as parameter test_result
 * \sa is_universal_statistical_test_accepted(),
 *     print_universal_statistical_test()
 */
struct test_params_t* universal_statistical_test(
	buffer_t buffer,
	size_t block_size,
	struct test_params_t* test_results,
	bool fixed_Q,
	int block_size_L
);

/**
 * \brief Test if the values are accepted.
 * The test result follow normal distribution within a small range, given
 * by the standard deviation that is specified by the rejection rate.
 * This function test if the previous calculated test result is between the
 * two borders t1 and t2 and return true if test is accepted or false if not.
 * \param test_result conterns the result of test calculated by
 *         universal_statistical_test
 * \param rejection_rate specifies the rejection rate. The rate should
 *         be between ]0, 0.5]
 * \return true if test is accepted 
 *         (and also set the field accepted in test_result)
 * \sa universal_statistical_test(), print_universal_statistical_test()
 */
bool is_universal_statistical_test_accepted(
	struct test_params_t *test_result,
	double rejection_rate
);

/** Print result of universal statistical test readable to screen.
 * \param test_result a pointer to a structure test_params_t with results
 *         of the test
 */
void print_universal_statistical_test(struct test_params_t *test_result);

#endif
